* ***************************************************
* *    KEYBEEP extension for keyboard click/beep    *
* * Separate beeps for CAPSLOCK on and CAPSLOCK off *
* *      on systems supporting use of sv_caps.      *
* ***************************************************
*
* Action depends on number of parameters in command.
*
* Four parameters:
* KEYBEEP off_duartion,off_pitch,on_duration,on_pitch
*
* Two parameters:
* KEYBEEP off_duration,off_pitch
*
* with one parameter, turns key beep on or off
* KEYBEEP 1      turns on
* KEYBEEP 0      turns off
*
* with no parameters, toggles on/off
* KEYBEEP
*
* N.B. Self-modifying code!
*
* Link into BASIC as extension procedure
*
        lea.l   exts,a1         ;point to list of extensions
        move.w  $110,a2         ;bp.init vector
        jmp     (a2)            ;do it and return to BASIC

* list of extension names and definitions
exts    dc.w    1               ;just 1 extension
        dc.w    keybeep-*       ;procedure is here
        dc.b    7,'KEYBEEP'     ;length of name and name of procedure
        dc.w    0               ;end of procedure definitions
        dc.w    0               ;no functions
        dc.w    0               ;end of function list

* new procedure entry point, action depends on how many parameters
keybeep move.w  $112,a2         ;ca.gtint to fetch parameters
        jsr     (a2)            ;fetch up to 4 parameters
        bne.s   error           ;oops, something went wrong...

        tst.w   d3              ;no parameters?
        beq.s   nopar           ;toggle scheduler loop task on/off

        cmpi.w  #1,d3           ;1 parameter (KEYBEEP 0/1 = off/on)
        beq.s   onepar          ;yes, switch on or off

        cmpi.w  #2,d3           ;2 parameters? KEYBEEP dur1,pitch1
        beq.s   twopar          ;yes, capslock off only

        cmpi.w  #4,d3           ;4 parameters?
        beq.s   fourpar         ;yes, KEYBEEP dur1,pitch1,dur2,pitch2

* error message return for anything else
err_bp  moveq   #-15,d0         ;bad parameter error
error   rts                     ;back to basic with error code in d0


* KEYBEEP - no parameter, so toggle existing state
nopar   lea     lstlink,a0      ;point to link long words
        tst.l   4(a0)           ;zero (=off)?
        beq.s   turnon          ;currently 0, so turn on key beep
        bra.s   turnoff         ;currently non-zero, so turn off key beep

* KEYBEEP 0 or KEYBEEP 1
* fetch the parameter and check if 0(off) or 1(on)
onepar  tst.w   0(a6,a1.l)      ;0=turn off key beep
        bne.s   turnon          ;1=turn on key beep

* turn off key beep service routine
turnoff lea.l   lstlink,a0      ;address of service routine
        tst.l   4(a0)           ;is it already off?
        beq.s   return1         ;yes it is off, so don't try to remove again!
        moveq   #$1f,d0         ;mt.rschd remove scheduler loop task
        trap    #1
        clr.l   (a0)+           ;zero first long word and point to second
        clr.l   (a0)            ;zero second long word of link area
return1 rts                     ;back to basic


* KEYBEEP off_duration,off_pitch,on_duration,on_pitch
fourpar lea.l   beepon,a3       ;where to store caps on key beep values
        movem.w 4(a6,a1.l),d6-d7 ;pitch and duration for caps on beep
        ror.w   #8,d6           ;duration byte order needs to be reversed
        move.w  d6,10(a3)       ;store duration for caps on beep
        move.b  d7,6(a3)        ;store pitch for caps on beep

* enter here for 2 parameter KEYBEEP off_duration,off_pitch
twopar  lea.l   beepoff,a3      ;where to store caps off key beep values
        movem.w 0(a6,a1.l),d6-d7 ;pitch and duration for caps off beep
        ror.w   #8,d6           ;duration byte order needs to be reversed
        move.w  d6,10(a3)       ;store duration for caps off key beep
        move.b  d7,6(a3)        ;store pitch for caps off key beep

* turn on key beep service routine
turnon  lea.l   lstlink,a0      ;point to the linked list long words
        tst.l   4(a0)           ;is it zero?
        bne.s   return2         ;already on, so no need to link in again
        lea.l   routine,a1      ;point to the service routine
        move.l  a1,4(a0)        ;store our service routine adress in link
        moveq   #$1e,d0         ;mt.lschd scheduler loop task
        trap    #1              ;link our service routine to the list
return2 rts                     ;back to basic

* this is the scheduler loop task itself
routine moveq   #0,d0           ;mt.inf, get system variables address
        trap    #1
        move.w  $8c(a0),d0      ;value of sv.ardel
        subq.w  #1,d0           ;auto repeat delay-1
        cmp.w   $90(a0),d0      ;compare to sv.arcnt
        bne.s   nosound         ;no need to play sound
        subq.w  #1,$90(a0)      ;sv.arcnt=sv.arcnt-1
        lea.l   inpstat,a3      ;pointer to BEEPING IPC command
        moveq   #$11,d0         ;mt.ipcom
        trap    #1              ;check if sound already playing
        btst    #1,d1           ;test BEEPING bit value returned
        bne.s   nosound         ;sound already playing
        tst.w   $88(a0)         ;caps lock off?
        bne.s   capson          ;caps lock is on
capsoff lea.l   beepoff,a3      ;address of sound block for caps off
        bra.s   sound           ;call the mt.ipcom trap
capson  lea.l   beepon,a3       ;address of sound block for caps on
sound   moveq   #$11,d0         ;mt.ipcom
        trap    #1              ;call mt.ipcom to play the sound
nosound rts                     ;service routine exit point

* this is the IPC command to test if sound already playing (like BEEPING)
inpstat dc.b    1               ;1 = report input status
        dc.b    0               ;0 parameters to send
        dc.l    0               ;ignored for this command
        dc.b    2               ;size of reply from IPC-send all bits
        dc.b    0               ;align to even

* this is the IPC command to play the key beep sound for CAPS OFF
beepoff dc.b    10              ;10 = command for sound
        dc.b    8               ;number of parameters to send to IPC
        dc.l    $0000aaaa       ;send all bits of each byte
        dc.b    30              ;pitch 1 value
        dc.b    0               ;pitch 2 value (not used)
        dc.w    0               ;grad_x value (not used)
        dc.w    $8000           ;duration 128 (bytes reverse order)
        dc.b    0               ;wrap+(16*grad_y) (not used)
        dc.b    0               ;fuzziness+(16*randomness)
        dc.b    1               ;no reply values
        dc.b    0               ;align to even

* this is the IPC command to play the key beep sound for CAPS ON
beepon  dc.b    10              ;10 = command for sound
        dc.b    8               ;number of parameters to send to IPC
        dc.l    $0000aaaa       ;send all bits of each byte
        dc.b    1               ;pitch 1 value
        dc.b    0               ;pitch 2 value (not used)
        dc.w    0               ;grad_x value (not used)
        dc.w    $8000           ;duration 128 (bytes reverse order)
        dc.b    0               ;wrap+(16*grad_y) (not used)
        dc.b    0               ;fuzziness+(16*randomness)
        dc.b    1               ;no reply values
        dc.b    0               ;align to even


* acheduler loop task routine linked list entry (2 long words)
lstlink dc.l    0               ;pointer to next link
        dc.l    0               ;address of service routine
